@@ -20,6 +20,7 @@
 import os
 import pkgutil
 import textwrap
+import urllib.request
 
 from typing import List
 from typing import Dict
@@ -32,16 +33,30 @@ from typing import Any
 import mako.template
 
 from ...optconf import Option
+from ...optconf.converters import as_string_list_choices
 
-from ...plugins.trackers import BaseTracker  # FIXME
+from ...plugins.trackers import BaseTracker
 
 from ...tfile import TorrentsDiff
 from ...tfile import Torrent
 
+from ... import web
+
 from .. import BasePlugin
 from .. import get_classes
 
 
+# =====
+STATUSES = {
+    "invalid":         "Invalid torrent",
+    "not_in_client":   "Torrent not in client",
+    "unknown":         "Unknown tracker type",
+    "affected":        "Updated",
+    "tracker_error":   "Tracker error",
+    "unhandled_error": "Unhandled error",
+}
+
+
 # =====
 def templated(name: str, built_in: bool=True, **kwargs: Any) -> str:
     if built_in:
@@ -56,7 +71,7 @@ def templated(name: str, built_in: bool=True, **kwargs: Any) -> str:
 
 
 # =====
-class _InnerUpdateResult(NamedTuple):
+class UpdateResult(NamedTuple):
     torrent: Optional[Torrent]
     tracker: Optional[BaseTracker]
     diff: TorrentsDiff
@@ -64,8 +79,6 @@ class _InnerUpdateResult(NamedTuple):
     err_msg: str
     tb_lines: List[str]
 
-
-class UpdateResult(_InnerUpdateResult):
     @staticmethod
     def new(
         torrent: Optional[Torrent]=None,
@@ -79,7 +92,7 @@ class UpdateResult(_InnerUpdateResult):
         return UpdateResult(
             torrent=torrent,
             tracker=tracker,
-            diff=(diff or TorrentsDiff.new()),
+            diff=(diff or TorrentsDiff()),
             err_name=err_name,
             err_msg=err_msg,
             tb_lines=(tb_lines or []),
@@ -90,10 +103,33 @@ ResultsType = Dict[str, Dict[str, UpdateResult]]
 
 
 class BaseConfetti(BasePlugin):
-    def __init__(self, timeout: float, retries: int, retries_sleep: float, **_: Any) -> None:  # pylint: disable=super-init-not-called
-        self._timeout = timeout
-        self._retries = retries
-        self._retries_sleep = retries_sleep
+    def __init__(self, **_: Any) -> None:  # pylint: disable=super-init-not-called
+        pass
+
+    def send_results(self, source: str, results: ResultsType) -> None:
+        raise NotImplementedError
+
+
+class WithWeb(BaseConfetti):  # pylint: disable=abstract-method
+    _SITE_RETRY_CODES: List[int] = []
+
+    def __init__(  # pylint: disable=super-init-not-called
+        self,
+        timeout: float,
+        retries: int,
+        retries_sleep: float,
+        user_agent: str,
+        proxy_url: str,
+        **_: Any,
+    ) -> None:
+
+        self.__timeout = timeout
+        self.__retries = retries
+        self.__retries_sleep = retries_sleep
+        self.__user_agent = user_agent
+        self.__proxy_url = proxy_url
+
+        self.__opener: Optional[urllib.request.OpenerDirector] = None
 
     @classmethod
     def get_options(cls) -> Dict[str, Option]:
@@ -101,20 +137,53 @@ class BaseConfetti(BasePlugin):
             "timeout":       Option(default=10.0, help="Network timeout"),
             "retries":       Option(default=5, help="Retries for failed attempts"),
             "retries_sleep": Option(default=1.0, help="Sleep interval between failed attempts"),
+            "user_agent":    Option(default="Mozilla/5.0", help="User-Agent for site"),
+            "proxy_url":     Option(default="", help="URL of HTTP/SOCKS4/SOCKS5 proxy"),
         }
 
-    def send_results(self, source: str, results: ResultsType) -> None:
-        raise NotImplementedError
+    def _init_opener(self) -> None:
+        assert not self.__opener
+        self.__opener = web.build_opener(self.__proxy_url)
+
+    def _read_url(
+        self,
+        url: str,
+        data: Optional[bytes]=None,
+        headers: Optional[Dict[str, str]]=None,
+    ) -> bytes:
+
+        assert self.__opener
+        headers = (headers or {})
+        headers.setdefault("User-Agent", self.__user_agent)
+        return web.read_url(
+            opener=self.__opener,
+            url=url,
+            data=data,
+            headers=headers,
+            timeout=self.__timeout,
+            retries=self.__retries,
+            retries_sleep=self.__retries_sleep,
+            retry_codes=self._SITE_RETRY_CODES,
+        )
 
 
-class WithProxy(BaseConfetti):  # pylint: disable=abstract-method
-    def __init__(self, proxy_url: str, **_: Any) -> None:  # pylint: disable=super-init-not-called
-        self._proxy_url = proxy_url
+class WithStatuses(BaseConfetti):  # pylint: disable=abstract-method
+    def __init__(  # pylint: disable=super-init-not-called
+        self,
+        statuses: List[str],
+        **_: Any,
+    ) -> None:
+
+        self._statuses = statuses
 
     @classmethod
     def get_options(cls) -> Dict[str, Option]:
         return {
-            "proxy_url": Option(default="", help="URL of HTTP/SOCKS4/SOCKS5 proxy"),
+            "statuses": Option(
+                default=["invalid", "not_in_client", "tracker_error", "unhandled_error", "affected"],
+                type=(lambda arg: as_string_list_choices(arg, list(STATUSES))),
+                help="Statuses to notifications",
+            ),
         }
 
 
